@@ -16,6 +16,9 @@
 /* Have memcpy() function. */
 #undef HAVE_MEMCPY
 
+/* Have strtoul() function. */
+#undef HAVE_STRTOUL
+
 /* How to retrieve the current working directory? */
 #undef HAVE_GETCWD
 #undef HAVE_GETWD
