@@ -641,6 +641,14 @@ afm_font_encoding (AFMFont font, AFMEncoding enc, unsigned int flags)
       apply_encoding (font, afm_88597_encoding, flags);
       break;
 
+    case AFM_ENCODING_ISO_8859_9:
+      apply_encoding (font, afm_88599_encoding, flags);
+      break;
+
+    case AFM_ENCODING_ISO_8859_10:
+      apply_encoding (font, afm_885910_encoding, flags);
+      break;
+
     case AFM_ENCODING_IBMPC:
       apply_encoding (font, afm_ibmpc_encoding, flags);
       break;
