@@ -180,6 +180,8 @@ typedef enum
   ENC_ISO_8859_4,
   ENC_ISO_8859_5,
   ENC_ISO_8859_7,
+  ENC_ISO_8859_9,
+  ENC_ISO_8859_10,
   ENC_ASCII,
   ENC_ASCII_FISE,
   ENC_ASCII_DKNO,
@@ -191,6 +193,16 @@ typedef enum
   ENC_PS
 } InputEncoding;
 
+struct encoding_registry_st
+{
+  char *names[3];
+  InputEncoding encoding;
+  int nl;
+  int bs;
+};
+
+typedef struct encoding_registry_st EncodingRegistry;
+
 typedef enum
 {
   LABEL_SHORT,
@@ -368,12 +380,16 @@ extern char *Fname;
 extern FontPoint Fpt;
 extern FontPoint default_Fpt;
 extern char *default_Fname;
+extern InputEncoding default_Fencoding;
 
 extern double font_widths[];
 extern char font_ctype[];
 extern int font_is_fixed;
 extern double font_bbox_lly;
 
+/* Known input encodings. */
+extern EncodingRegistry encodings[];
+
 /* Options. */
 
 extern char *printer;
@@ -446,10 +462,13 @@ extern char *toc_fmt_string;
 extern unsigned int file_align;
 extern int slicing;
 extern unsigned int slice;
-extern char states_path[];
-extern char states_color_model[];
+
+extern char states_binary[];
+extern int states_color;
 extern char states_config_file[];
-extern char states_highlight_level[];
+extern char states_highlight_style[];
+extern char states_path[];
+
 extern unsigned int nup;
 extern unsigned int nup_rows;
 extern unsigned int nup_columns;
@@ -570,13 +589,15 @@ void tilde_subst ___P ((char *from, char *to));
 double parse_float ___P ((char *string, int units, int horizontal));
 
 /*
- * Parse font spec <spec> and return font's name and size in variables
- * <name_return> and <size_return>.  Returns 1 if <spec> was a valid
- * font spec or 0 otherwise.  Returned name <name_return> is allocated
- * with xcalloc() and must be freed by caller.
+ * Parse font spec <spec> and return font's name, size, and encoding
+ * in variables <name_return>, <size_return>, and <encoding_return>.
+ * Returns 1 if <spec> was a valid font spec or 0 otherwise.  Returned
+ * name <name_return> is allocated with xcalloc() and must be freed by
+ * caller.
  */
 int parse_font_spec ___P ((char *spec, char **name_return,
-			   FontPoint *size_return));
+			   FontPoint *size_return,
+			   InputEncoding *encoding_return));
 
 /*
  * Read body font's character widths and character codes from AFM files.
