@@ -235,6 +235,7 @@ char *Fname = "Courier";
 FontPoint Fpt = {10.0, 10.0};
 FontPoint default_Fpt;		/* Point size of the original font. */
 char *default_Fname;		/* Name of the original font. */
+InputEncoding default_Fencoding; /* The encoding of the original font. */
 int user_body_font_defined = 0;	/* Has user defined new body font? */
 
 double font_widths[256];	/* Width array for body font. */
@@ -500,9 +501,9 @@ int interpret_formfeed = 1;
 int pass_through = 0;
 
 /*
- * --color[=COLOR]
+ * --color[=bool]
  *
- * Create color output with StatesColorModel COLOR.
+ * Create color output with states?
  */
 
 /*
@@ -722,30 +723,40 @@ char queue_param[16];
 char spooler_command[256];
 
 /*
- * StatesColorModel: model
+ * StatesBinary: path
  *
- * Color model for states.
+ * An absolute path to the `states' binary.
  */
-char states_color_model[50];
+
+char states_binary[1024];
+
+/*
+ * StatesColor: bool
+ *
+ * Should the States program generate color outputs.
+ */
+int states_color = 0;
 
 /*
  * StatesConfigFile: file
  *
- * Read states' configuration from file <file>.
+ * The name of the states' configuration file.  It is looked up from
+ * the states_path.
  */
 char states_config_file[256];
 
 /*
- * StatesHighlightLevel: level
+ * StatesHighlightStyle: style
  *
- * HighlightLevel for states.
+ * The highlight style.
  */
-char states_highlight_level[50];
+char states_highlight_style[50];
 
 /*
  * StatesPath: path
  *
- * Define the path for the states program.
+ * Define the path for the states program.  The states program will
+ * lookup its state definition files from this path.
  */
 char states_path[1024];
 
@@ -755,6 +766,31 @@ double line_highlight_gray = 1.0;
 /* ^@bggray{GRAY}, set the text background gray. */
 double bggray = 1.0;
 
+EncodingRegistry encodings[] =
+{
+  {{"88591", "latin1", NULL},		ENC_ISO_8859_1,		'\n', 8},
+  {{"88592", "latin2", NULL},		ENC_ISO_8859_2,		'\n', 8},
+  {{"88593", "latin3", NULL},		ENC_ISO_8859_3,		'\n', 8},
+  {{"88594", "latin4", NULL},		ENC_ISO_8859_4,		'\n', 8},
+  {{"88595", "cyrillic", NULL},		ENC_ISO_8859_5,		'\n', 8},
+  {{"88597", "greek", NULL},		ENC_ISO_8859_7,		'\n', 8},
+  {{"88599", "latin5", NULL},		ENC_ISO_8859_9,		'\n', 8},
+  {{"885910", "latin6", NULL},		ENC_ISO_8859_10,	'\n', 8},
+  {{"ascii", NULL, NULL},		ENC_ASCII, 		'\n', 8},
+  {{"asciifise", "asciifi", "asciise"},	ENC_ASCII_FISE,		'\n', 8},
+  {{"asciidkno", "asciidk", "asciino"},	ENC_ASCII_DKNO,		'\n', 8},
+  {{"ibmpc", "pc", "dos"},		ENC_IBMPC, 		'\n', 8},
+  {{"mac", NULL, NULL},			ENC_MAC, 		'\r', 8},
+  {{"vms", NULL, NULL},			ENC_VMS, 		'\n', 8},
+  {{"hp8", NULL, NULL},			ENC_HP8,		'\n', 8},
+  {{"koi8", NULL, NULL},		ENC_KOI8,		'\n', 8},
+  {{"ps", "PS", NULL},			ENC_PS, 		'\n', 8},
+  {{"pslatin1", "ISOLatin1Encoding", NULL},	ENC_ISO_8859_1,	'\n', 8},
+
+  {{NULL, NULL, NULL}, 0, 0, 0},
+};
+
+
 /*
  * Static variables.
  */
@@ -828,6 +864,7 @@ static struct option long_options[] =
   {"printer-options",		required_argument,	0, 139},
   {"rotate-even-pages",		no_argument,		0, 150},
   {"slice",			required_argument,	0, 140},
+  {"style",			required_argument,	0, 151},
   {"toc",			no_argument,		&toc, 1},
   {"word-wrap",			no_argument,		0, 147},
   {"ul-angle",			required_argument,	0, 132},
@@ -840,35 +877,6 @@ static struct option long_options[] =
 };
 
 
-static struct
-{
-  char *names[3];
-  InputEncoding encoding;
-  int nl;
-  int bs;
-} encodings[] =
-  {
-    {{"88591", "latin1", NULL},		ENC_ISO_8859_1,		'\n', 8},
-    {{"88592", "latin2", NULL},		ENC_ISO_8859_2,		'\n', 8},
-    {{"88593", "latin3", NULL},		ENC_ISO_8859_3,		'\n', 8},
-    {{"88594", "latin4", NULL},		ENC_ISO_8859_4,		'\n', 8},
-    {{"88595", "cyrillic", NULL},	ENC_ISO_8859_5,		'\n', 8},
-    {{"88597", "greek", NULL},		ENC_ISO_8859_7,		'\n', 8},
-    {{"ascii", NULL, NULL},		ENC_ASCII, 		'\n', 8},
-    {{"asciifise", "asciifi", "asciise"},	ENC_ASCII_FISE,	'\n', 8},
-    {{"asciidkno", "asciidk", "asciino"},	ENC_ASCII_DKNO,	'\n', 8},
-    {{"ibmpc", "pc", "dos"},		ENC_IBMPC, 		'\n', 8},
-    {{"mac", NULL, NULL},		ENC_MAC, 		'\r', 8},
-    {{"vms", NULL, NULL},		ENC_VMS, 		'\n', 8},
-    {{"hp8", NULL, NULL},		ENC_HP8,		'\n', 8},
-    {{"koi8", NULL, NULL},		ENC_KOI8,		'\n', 8},
-    {{"ps", "PS", NULL},		ENC_PS, 		'\n', 8},
-    {{"pslatin1", "ISOLatin1Encoding", NULL},	ENC_ISO_8859_1,	'\n', 8},
-
-    {{NULL, NULL, NULL}, 0, 0, 0},
-  };
-
-
 /*
  * Global functions.
  */
@@ -903,6 +911,7 @@ main (int argc, char *argv[])
   *cp = ' ';
 
   /* Create the default TOC format string.  Wow, this is cool! */
+  /* xgettext:no-c-format */
   toc_fmt_string = _("$3v $-40N $3% pages $4L lines  $E $C");
 
   /* Internationalization. */
@@ -948,15 +957,20 @@ main (int argc, char *argv[])
   if (cp)
     enscript_library = cp;
 
-  /* Defaults for the states filter. */
-  strcpy (states_color_model, "blackwhite");
-  sprintf (states_config_file, "%s/enscript.st", enscript_library);
-  strcpy (states_highlight_level, "heavy");
-  strcpy (states_path, "states"); /* Take it from the user path. */
-
   /* Fill up build-in libpath. */
-  sprintf (libpath, "%s%c%s/.enscript", enscript_library, PATH_SEPARATOR,
-	   passwd->pw_dir);
+  cp = getenv ("HOME");
+  if (cp == NULL)
+    cp = passwd->pw_dir;
+  sprintf (libpath, "%s%c%s/.enscript", enscript_library, PATH_SEPARATOR, cp);
+
+  /* Defaults for the states filter. */
+  strcpy (states_binary, "states"); /* Take it from the user path. */
+  sprintf (states_config_file, "%s/hl/enscript.st", enscript_library);
+  strcpy (states_highlight_style, "emacs");
+
+  /* The <cp> holds the user's home directory. */
+  sprintf (states_path, "%s/.enscript%c%s/hl", cp, PATH_SEPARATOR,
+	   enscript_library);
 
   /* Initialize resource sets. */
   res_fonts = strhash_init ();
@@ -1075,6 +1089,7 @@ main (int argc, char *argv[])
   default_Fpt.w = Fpt.w;
   default_Fpt.h = Fpt.h;
   default_Fname = Fname;
+  default_Fencoding = encoding;
 
   /* Register that document uses at least these fonts. */
   strhash_put (res_fonts, Fname, strlen (Fname) + 1, NULL, NULL);
@@ -1335,9 +1350,13 @@ name             width\theight\tllx\tlly\turx\tury\n\
   if (help_pretty_print)
     {
       /* Create description with states. */
+      printf (_("Highlighting is supported for the following languages and file formats:\n\n"));
+      fflush (stdout);
+
       sprintf (pp_filter,
-	       "%s -f \"%s\" -s describe_languages \"%s\"",
-	       states_path, states_config_file, states_config_file);
+	       "%s -f \"%s\" -p \"%s\" -s describe_languages %s/hl/*.st",
+	       states_binary, states_config_file, states_path,
+	       enscript_library);
       system (pp_filter);
       exit (0);
     }
@@ -1379,12 +1398,13 @@ name             width\theight\tllx\tlly\turx\tury\n\
 
       /* Create states command. */
       cp = (char *) xmalloc (j);
-      sprintf (cp, "%s -f \"%s\" %s%s -Dcolormodel=%s -Dhl_level=%s -Dlanguage=%s -Dnum_input_files=%d -Ddocument_title=\"%s\" -Dtoc=%d",
-	       states_path, states_config_file,
+      sprintf (cp, "%s -f \"%s\" -p \"%s\" %s %s%s -Dcolor=%d -Dstyle=%s -Dlanguage=%s -Dnum_input_files=%d -Ddocument_title=\"%s\" -Dtoc=%d",
+	       states_binary, states_config_file, states_path,
+	       verbose > 0 ? "-v" : "",
 	       start_state ? "-s" : "",
 	       start_state ? start_state : "",
-	       states_color_model,
-	       states_highlight_level,
+	       states_color,
+	       states_highlight_style,
 	       output_language,
 	       optind == argc ? 1 : argc - optind,
 	       title,
@@ -1414,13 +1434,13 @@ name             width\theight\tllx\tlly\turx\tury\n\
 	{
 	  /* Create pretty-print input filter. */
 	  sprintf (pp_filter,
-		   "%s -f \"%s\" %s%s -Dcolormodel=%s -Dhl_level=%s -Dfont_spec=%s@%g/%g \"%%s\"",
-		   states_path,
-		   states_config_file,
+		   "%s -f \"%s\" -p \"%s\" %s %s%s -Dcolor=%d -Dstyle=%s -Dfont_spec=%s@%g/%g \"%%s\"",
+		   states_binary, states_config_file, states_path,
+		   verbose > 0 ? "-v" : "",
 		   pp_start_state ? "-s " : "",
 		   pp_start_state ? pp_start_state : "",
-		   states_color_model,
-		   states_highlight_level,
+		   states_color,
+		   states_highlight_style,
 		   Fname, Fpt.w, Fpt.h);
 	  input_filter = pp_filter;
 	  input_filter_stdin = "-";
@@ -1912,13 +1932,13 @@ handle_options (int argc, char *argv[])
 	  break;
 
 	case 'f':		/* font */
-	  if (!parse_font_spec (optarg, &Fname, &Fpt))
+	  if (!parse_font_spec (optarg, &Fname, &Fpt, NULL))
 	    FATAL ((stderr, _("malformed font spec: %s"), optarg));
 	  user_body_font_defined = 1;
 	  break;
 
 	case 'F':		/* header font */
-	  if (!parse_font_spec (optarg, &HFname, &HFpt))
+	  if (!parse_font_spec (optarg, &HFname, &HFpt, NULL))
 	    FATAL ((stderr, _("malformed font spec: %s"), optarg));
 	  break;
 
@@ -2097,7 +2117,7 @@ handle_options (int argc, char *argv[])
 	  break;
 
 	case 128:		/* underlay font */
-	  if (!parse_font_spec (optarg, &ul_font, &ul_ptsize))
+	  if (!parse_font_spec (optarg, &ul_font, &ul_ptsize, NULL))
 	    FATAL ((stderr, _("malformed font spec: %s"), optarg));
 	  break;
 
@@ -2160,11 +2180,11 @@ handle_options (int argc, char *argv[])
 	  help_pretty_print = 1;
 	  break;
 
-	case 142:		/* States color model. */
+	case 142:		/* States color? */
 	  if (optarg == NULL)
-	    strcpy (states_color_model, "emacs");
+	    states_color = 1;
 	  else
-	    strcpy (states_color_model, optarg);
+	    states_color = atoi (optarg);
 	  break;
 
 	case 143:		/* mark-wrapped-lines */
@@ -2204,6 +2224,10 @@ handle_options (int argc, char *argv[])
 	  rotate_even_pages = 1;
 	  break;
 
+	case 151:		/* highlight style */
+	  strcpy (states_highlight_style, optarg);
+	  break;
+
 	case '?':		/* Errors found during getopt_long(). */
 	option_error:
 	  fprintf (stderr, _("Try `%s --help' for more information.\n"),
@@ -2392,7 +2416,7 @@ Mandatory arguments to long options are mandatory for short options too.\n\
                              without any modifications\n"));
 
   printf (_("Long-only options:\n\
-  --color[=COLOR]            set output color model to COLOR\n\
+  --color[=bool]             create color outputs with states\n\
   --download-font=NAME       download font NAME\n\
   --filter-stdin=NAME        specify how stdin is shown to the input filter\n\
   --h-column-height=HEIGHT   set the horizontal column height to HEIGHT\n\
@@ -2419,6 +2443,7 @@ Mandatory arguments to long options are mandatory for short options too.\n\
 
   printf (_("\
   --slice=NUM                print vertical slice NUM\n\
+  --style=STYLE              use highlight style STYLE\n\
   --toc                      print table of contents\n\
   --ul-angle=ANGLE           set underlay text's angle to ANGLE\n\
   --ul-font=NAME             print underlays with font NAME\n\
