@@ -77,6 +77,7 @@ typedef enum
   tNEWPAGE,
   tFONT,
   tCOLOR,
+  tBGCOLOR,
   tPS
 } TokenType;
 
@@ -87,6 +88,7 @@ typedef enum
   ESC_EPSF,
   ESC_FONT,
   ESC_COLOR,
+  ESC_BGCOLOR,
   ESC_NEWPAGE,
   ESC_SETFILENAME,
   ESC_SETPAGENUMBER,
@@ -126,10 +128,12 @@ struct gs_token_st
 	  int pipe;		/* Is <fp> opened to pipe?  */
 	} epsf;
       Color color;
+      Color bgcolor;
       struct
 	{
 	  char name[512];
 	  FontPoint size;
+	  InputEncoding encoding;
 	} font;
       char filename[512];
     } u;
@@ -241,6 +245,7 @@ static int user_fontp = 0;
 /* The user ^@font{}-defined font. */
 static char user_font_name[256];
 static FontPoint user_font_pt;
+static InputEncoding user_font_encoding;
 
 /* Is ^@color{}-defined color active? */
 static int user_colorp = 0;
@@ -248,6 +253,12 @@ static int user_colorp = 0;
 /* The user ^@color{}-defined color. */
 static Color user_color;
 
+/* Is ^@bgcolor{}-defined color active? */
+static int user_bgcolorp = 0;
+
+/* The user ^@bgcolor{}-defined color. */
+static Color user_bgcolor;
+
 /* The last linenumber printed by print_line_number(). */
 static unsigned int print_line_number_last;
 
@@ -687,11 +698,6 @@ process_file (char *fname_arg, InputStream *is)
 		  page_clear = 0;
 		}
 
-	      /* Print line numbers if needed. */
-	      if (line_numbers && line_column == 0 && token.type != tFORMFEED)
-		print_line_number (lx, y, linenumber_space, linenumber_margin,
-				   current_file_linenum);
-
 	      /* Print line highlight. */
 	      if (line_column == 0 && line_highlight_gray < 1.0)
 		OUTPUT ((cofp, "%g %g %g %g %g line_highlight\n",
@@ -700,6 +706,11 @@ process_file (char *fname_arg, InputStream *is)
 			 linewidth, Fpt.h + baselineskip,
 			 line_highlight_gray));
 
+	      /* Print line numbers if needed. */
+	      if (line_numbers && line_column == 0 && token.type != tFORMFEED)
+		print_line_number (lx, y, linenumber_space, linenumber_margin,
+				   current_file_linenum);
+
 	      /* Check rest of tokens. */
 	      switch (token.type)
 		{
@@ -745,6 +756,17 @@ process_file (char *fname_arg, InputStream *is)
 				   bggray,
 				   token.u.str));
 			}
+		      else if (user_bgcolorp)
+			{
+			  OUTPUT ((cofp, "%g %g %g %g %g %g %g (%s) bgcs\n",
+				   x, y, Fpt.h + baselineskip,
+				   baselineskip
+				   - (font_bbox_lly * Fpt.h / UNITS_PER_POINT),
+				   user_bgcolor.r,
+				   user_bgcolor.g,
+				   user_bgcolor.b,
+				   token.u.str));
+			}
 		      else
 			{
 			  OUTPUT ((cofp, "%g %g M\n(%s) s\n", x, y,
@@ -911,6 +933,7 @@ large for page\n"),
 		      Fpt.w = default_Fpt.w;
 		      Fpt.h = default_Fpt.h;
 		      Fname = default_Fname;
+		      encoding = default_Fencoding;
 		      OUTPUT ((cofp, "/F-gs-font %g %g SF\n", Fpt.w, Fpt.h));
 		      user_fontp = 0;
 		    }
@@ -919,17 +942,26 @@ large for page\n"),
 		      strhash_put (res_fonts, token.u.font.name,
 				   strlen (token.u.font.name) + 1,
 				   NULL, NULL);
-		      OUTPUT ((cofp, "/%s %g %g SUF\n", token.u.font.name,
-			       token.u.font.size.w, token.u.font.size.h));
+		      if (token.u.font.encoding == default_Fencoding)
+			OUTPUT ((cofp, "/%s %g %g SUF\n", token.u.font.name,
+				 token.u.font.size.w, token.u.font.size.h));
+		      else if (token.u.font.encoding == ENC_PS)
+			OUTPUT ((cofp, "/%s %g %g SUF_PS\n", token.u.font.name,
+				 token.u.font.size.w, token.u.font.size.h));
+		      else
+			FATAL ((stderr,
+				_("user font encoding can be only the system's default or `ps'")));
 
 		      strcpy (user_font_name, token.u.font.name);
 		      user_font_pt.w = token.u.font.size.w;
 		      user_font_pt.h = token.u.font.size.h;
+		      user_font_encoding = token.u.font.encoding;
 		      user_fontp = 1;
 
 		      Fpt.w = user_font_pt.w;
 		      Fpt.h = user_font_pt.h;
 		      Fname = user_font_name;
+		      encoding = user_font_encoding;
 		    }
 		  MESSAGE (2, (stderr, "%s %g/%gpt\n", Fname, Fpt.w, Fpt.h));
 		  read_font_info ();
@@ -971,6 +1003,29 @@ large for page\n"),
 		    }
 		  break;
 
+		case tBGCOLOR:
+		  /* Select a new background color. */
+		  MESSAGE (2, (stderr, "^@bgcolor{%f %f %f}\n",
+			       token.u.color.r,
+			       token.u.color.g,
+			       token.u.color.b));
+
+		  if (token.u.color.r == token.u.color.g
+		      && token.u.color.g == token.u.color.b
+		      && token.u.color.b == 1.0)
+		    {
+		      /* Select the default bgcolor (white). */
+		      user_bgcolorp = 0;
+		    }
+		  else
+		    {
+		      user_bgcolor.r = token.u.color.r;
+		      user_bgcolor.g = token.u.color.g;
+		      user_bgcolor.b = token.u.color.b;
+		      user_bgcolorp = 1;
+		    }
+		  break;
+
 		case tSETFILENAME:
 		  strcpy (fname, token.u.filename);
 		  break;
@@ -1081,6 +1136,7 @@ static struct
     {"epsf", 		ESC_EPSF},
     {"font", 		ESC_FONT},
     {"color",		ESC_COLOR},
+    {"bgcolor",		ESC_BGCOLOR},
     {"newpage",		ESC_NEWPAGE},
     {"ps",		ESC_PS},
     {"setfilename",	ESC_SETFILENAME},
@@ -1361,7 +1417,8 @@ read_special_escape (InputStream *is, Token *token)
 	  else
 	    {
 	      if (!parse_font_spec (token->u.font.name, &cp,
-				    &token->u.font.size))
+				    &token->u.font.size,
+				    &token->u.font.encoding))
 		FATAL ((stderr, _("malformed font spec for ^@font escape: %s"),
 			token->u.font.name));
 
@@ -1372,12 +1429,18 @@ read_special_escape (InputStream *is, Token *token)
 	  break;
 
 	case ESC_COLOR:
+	case ESC_BGCOLOR:
 	  /* Check for the default color. */
 	  if (strcmp (buf, "default") == 0)
 	    {
-	      token->u.color.r = 0;
-	      token->u.color.g = 0;
-	      token->u.color.b = 0;
+	      double val = 0;
+
+	      if (escapes[e].escape == ESC_BGCOLOR)
+		val = 1;
+
+	      token->u.color.r = val;
+	      token->u.color.g = val;
+	      token->u.color.b = val;
 	    }
 	  else
 	    {
@@ -1392,7 +1455,9 @@ read_special_escape (InputStream *is, Token *token)
 		case 0:
 		case 2:
 		  FATAL ((stderr,
-			  _("malformed color spec for ^@color escape: %s"),
+			  _("malformed color spec for ^@%s escape: %s"),
+			  escapes[e].escape == ESC_COLOR
+			  ? "color" : "bgcolor",
 			  buf));
 		  break;
 
@@ -1405,7 +1470,10 @@ read_special_escape (InputStream *is, Token *token)
 		  break;
 		}
 	    }
-	  token->type = tCOLOR;
+	  if (escapes[e].escape == ESC_COLOR)
+	    token->type = tCOLOR;
+	  else
+	    token->type = tBGCOLOR;
 	  break;
 
 	case ESC_SHADE:
@@ -2054,7 +2122,13 @@ dump_ps_page_header (char *fname, int empty)
 
   /* Do we have a pending ^@font{} font? */
   if (user_fontp)
-    OUTPUT ((cofp, "/%s %g %g SUF\n", Fname, Fpt.w, Fpt.h));
+    {
+      if (encoding == default_Fencoding)
+	OUTPUT ((cofp, "/%s %g %g SUF\n", Fname, Fpt.w, Fpt.h));
+      else
+	/* This must be the case. */
+	OUTPUT ((cofp, "/%s %g %g SUF_PS\n", Fname, Fpt.w, Fpt.h));
+    }
 
   /* Dump user defined strings. */
   if (count_key_value_set (user_strings) > 0)
@@ -2564,6 +2638,7 @@ print_line_number (double x, double y, double space, double margin,
   int i;
   char *saved_Fname = "";
   FontPoint saved_Fpt;
+  InputEncoding saved_Fencoding;
 
   saved_Fpt.w = 0.0;
   saved_Fpt.h = 0.0;
@@ -2579,10 +2654,12 @@ print_line_number (double x, double y, double space, double margin,
       saved_Fname = Fname;
       saved_Fpt.w = Fpt.w;
       saved_Fpt.h = Fpt.h;
+      saved_Fencoding = encoding;
 
       Fname = default_Fname;
       Fpt.w = default_Fpt.w;
       Fpt.h = default_Fpt.h;
+      encoding = default_Fencoding;
 
       OUTPUT ((cofp, "/F-gs-font %g %g SF\n", Fpt.w, Fpt.h));
       read_font_info ();
@@ -2602,6 +2679,7 @@ print_line_number (double x, double y, double space, double margin,
       Fname = saved_Fname;
       Fpt.w = saved_Fpt.w;
       Fpt.h = saved_Fpt.h;
+      encoding = saved_Fencoding;
 
       OUTPUT ((cofp, "/%s %g %g SUF\n", Fname, Fpt.w, Fpt.h));
       read_font_info ();
