@@ -333,11 +333,17 @@ read_config (char *path, char *file)
 	  CHECK_TOKEN ();
 	  strcpy (spooler_command, token2);
 	}
-      else if (MATCH (token, "StatesColorModel:"))
+      else if (MATCH (token, "StatesBinary:"))
 	{
 	  token2 = GET_TOKEN (NULL);
 	  CHECK_TOKEN ();
-	  strcpy (states_color_model, token2);
+	  strcpy (states_binary, token2);
+	}
+      else if (MATCH (token, "StatesColor:"))
+	{
+	  token2 = GET_TOKEN (NULL);
+	  CHECK_TOKEN ();
+	  states_color = atoi (token2);
 	}
       else if (MATCH (token, "StatesConfigFile:"))
 	{
@@ -345,11 +351,11 @@ read_config (char *path, char *file)
 	  CHECK_TOKEN ();
 	  strcpy (states_config_file, token2);
 	}
-      else if (MATCH (token, "StatesHighlightLevel:"))
+      else if (MATCH (token, "StatesHighlightStyle:"))
 	{
 	  token2 = GET_TOKEN (NULL);
 	  CHECK_TOKEN ();
-	  strcpy (states_highlight_level, token2);
+	  strcpy (states_highlight_style, token2);
 	}
       else if (MATCH (token, "StatesPath:"))
 	{
@@ -387,7 +393,7 @@ read_config (char *path, char *file)
 	{
 	  token2 = GET_TOKEN (NULL);
 	  CHECK_TOKEN ();
-	  if (!parse_font_spec (token2, &ul_font, &ul_ptsize))
+	  if (!parse_font_spec (token2, &ul_font, &ul_ptsize, NULL))
 	    CFG_FATAL ((stderr, _("malformed font spec: %s"), token2));
 	}
       else if (MATCH (token, "UnderlayGray:"))
@@ -672,10 +678,23 @@ paste_file (char *name, char *suffix)
 
 
 int
-parse_font_spec (char *spec, char **name_return, FontPoint *size_return)
+parse_font_spec (char *spec_a, char **name_return, FontPoint *size_return,
+		 InputEncoding *encoding_return)
 {
-  int i;
+  int i, j;
   char *cp, *cp2;
+  char *spec;
+  char *encp;
+
+  spec = xstrdup (spec_a);
+
+  /* Check for the `namesize:encoding' format. */
+  encp = strrchr (spec, ':');
+  if (encp)
+    {
+      *encp = '\0';
+      encp++;
+    }
 
   /* The `name@ptsize' format? */
   cp = strchr (spec, '@');
@@ -683,8 +702,11 @@ parse_font_spec (char *spec, char **name_return, FontPoint *size_return)
     {
       i = cp - spec;
       if (cp[1] == '\0')
-	/* No ptsize after '@'. */
-	return 0;
+	{
+	  /* No ptsize after '@'. */
+	  xfree (spec);
+	  return 0;
+	}
       cp++;
 
       /* Check pt/pt format. */
@@ -703,12 +725,18 @@ parse_font_spec (char *spec, char **name_return, FontPoint *size_return)
       /* Old `nameptsize' format. */
       i = strlen (spec) - 1;
       if (i <= 0 || !ISNUMBERDIGIT (spec[i]))
-	return 0;
+	{
+	  xfree (spec);
+	  return 0;
+	}
 
       for (i--; i >= 0 && ISNUMBERDIGIT (spec[i]); i--)
 	;
       if (i < 0)
-	return 0;
+	{
+	  xfree (spec);
+	  return 0;
+	}
 
       /* Check pt/pt format. */
       if (spec[i] == '/')
@@ -718,7 +746,10 @@ parse_font_spec (char *spec, char **name_return, FontPoint *size_return)
 	  for (i--; i >= 0 && ISNUMBERDIGIT (spec[i]); i--)
 	    ;
 	  if (i < 0)
-	    return 0;
+	    {
+	      xfree (spec);
+	      return 0;
+	    }
 	  i++;
 	  size_return->w = atof (spec + i);
 	}
@@ -732,8 +763,49 @@ parse_font_spec (char *spec, char **name_return, FontPoint *size_return)
   *name_return = (char *) xcalloc (1, i + 1);
   strncpy (*name_return, spec, i);
 
-  MESSAGE (2, (stderr, "parse_font_spec(): name=%.*s, size=%g/%g\n", i,
-	       *name_return, size_return->w, size_return->h));
+  /* Check the input encoding. */
+  if (encp)
+    {
+      int found = 0;
+
+      if (encoding_return == NULL)
+	{
+	  /* We don't allow it here. */
+	  xfree (spec);
+	  return 0;
+	}
+
+      for (i = 0; !found && encodings[i].names[0]; i++)
+	for (j = 0; j < 3; j++)
+	  if (encodings[i].names[j] != NULL && MATCH (encodings[i].names[j],
+						      encp))
+	    {
+	      /* Found a match. */
+	      *encoding_return = encodings[i].encoding;
+	      encp = encodings[i].names[0];
+	      found = 1;
+	      break;
+	    }
+
+      if (!found)
+	{
+	  xfree (spec);
+	  return 0;
+	}
+    }
+  else
+    {
+      /* The spec didn't contain the encoding part.  Use our global default. */
+      encp = encoding_name;
+      if (encoding_return)
+	*encoding_return = encoding;
+    }
+  xfree (spec);
+
+  MESSAGE (2, (stderr,
+	       "parse_font_spec(): name=%.*s, size=%g/%g, encoding=%s\n", i,
+	       *name_return, size_return->w, size_return->h,
+	       encp));
 
   if (size_return->w < 0.0 && size_return->h < 0.0)
     MESSAGE (0, (stderr, _("%s: warning: font size is negative\n"), program));
@@ -764,7 +836,7 @@ read_font_info (void)
     enc_flags = AFM_ENCODE_ACCEPT_COMPOSITES;
 
   /* Open font */
-  sprintf (fkey, "%s@%f", Fname, Fpt.w);
+  sprintf (fkey, "%s@%f:%d", Fname, Fpt.w, encoding);
   if (!strhash_get (afm_info_cache, fkey, strlen (fkey), (void **) &font_info))
     {
       AFMError error;
@@ -830,6 +902,16 @@ read_font_info (void)
 					enc_flags);
 	      break;
 
+	    case ENC_ISO_8859_9:
+	      (void) afm_font_encoding (font, AFM_ENCODING_ISO_8859_9,
+					enc_flags);
+	      break;
+
+	    case ENC_ISO_8859_10:
+	      (void) afm_font_encoding (font, AFM_ENCODING_ISO_8859_10,
+					enc_flags);
+	      break;
+
 	    case ENC_ASCII:
 	      (void) afm_font_encoding (font, AFM_ENCODING_ASCII, enc_flags);
 	      break;
