@@ -1,6 +1,6 @@
 /*
  * General helper utilities.
- * Copyright (c) 1997 Markku Rossi.
+ * Copyright (c) 1997-1998 Markku Rossi.
  *
  * Author: Markku Rossi <mtr@iki.fi>
  */
@@ -95,6 +95,7 @@ node_alloc (type)
   n->type = type;
   n->refcount = 1;
   n->linenum = linenum;
+  n->filename = yyin_name;
 
   if (type == nREGEXP)
     n->u.re.compiled.fastmap = xmalloc (256);
@@ -112,6 +113,7 @@ node_copy (n)
 
   n2 = node_alloc (n->type);
   n2->linenum = n->linenum;
+  n2->filename = n->filename;
 
   switch (n->type)
     {
@@ -258,7 +260,7 @@ compile_regexp (re)
     {
       fprintf (stderr,
 	       _("%s:%d: couldn't compile regular expression \"%s\": %s\n"),
-	       defs_file, re->linenum, re->u.re.data, msg);
+	       re->filename, re->linenum, re->u.re.data, msg);
       exit (1);
     }
 
@@ -283,6 +285,7 @@ mk_stmt (type, arg1, arg2, arg3, arg4)
   stmt = (Stmt *) xcalloc (1, sizeof (*stmt));
   stmt->type = type;
   stmt->linenum = linenum;
+  stmt->filename = yyin_name;
 
   switch (type)
     {
@@ -335,6 +338,7 @@ mk_expr (type, arg1, arg2, arg3)
   expr = (Expr *) xcalloc (1, sizeof (*expr));
   expr->type = type;
   expr->linenum = linenum;
+  expr->filename = yyin_name;
 
   switch (type)
     {
@@ -425,23 +429,33 @@ cons (car, cdr)
 
 
 void
-define_state (sym, rules)
+define_state (sym, super, rules)
      Node *sym;
+     Node *super;
      List *rules;
 {
-  void *old_rules;
+  void *old_state;
   char msg[512];
+  State *state;
 
-  if (!strhash_put (ns_states, sym->u.sym, strlen (sym->u.sym), rules,
-		    &old_rules))
+  state = (State *) xcalloc (1, sizeof (*state));
+  state->name = xstrdup (sym->u.sym);
+  state->rules = rules;
+
+  if (super)
+    state->super_name = xstrdup (super->u.sym);
+
+  if (!strhash_put (ns_states, sym->u.sym, strlen (sym->u.sym), state,
+		    &old_state))
     {
       fprintf (stderr, _("%s: ouf of memory"), program);
       exit (1);
     }
-  if (old_rules)
+  if (old_state)
     {
       sprintf (msg, _("warning: redefining state `%s'"), sym->u.sym);
       yyerror (msg);
+      /* Yes, we leak memory here. */
     }
 }
 
@@ -451,9 +465,10 @@ define_state (sym, rules)
  */
 
 static void
-define_sub (sym, args_body, linenum)
+define_sub (sym, args_body, filename, linenum)
      Node *sym;
      Cons *args_body;
+     char *filename;
      unsigned int linenum;
 {
   void *old_data;
@@ -466,16 +481,17 @@ define_sub (sym, args_body, linenum)
     }
   if (old_data && warning_level >= WARN_ALL)
     fprintf (stderr, _("%s:%d: warning: redefining subroutine `%s'\n"),
-	     defs_file, linenum, sym->u.sym);
+	     filename, linenum, sym->u.sym);
 }
 
 extern unsigned int current_linenum;
 
 static Node *
-lookup_var (env, ns, sym, linenum)
+lookup_var (env, ns, sym, filename, linenum)
      Environment *env;
      StringHashPtr ns;
      Node *sym;
+     char *filename;
      unsigned int linenum;
 {
   Node *n;
@@ -552,7 +568,7 @@ lookup_var (env, ns, sym, linenum)
 
   /* Undefined variable. */
   fprintf (stderr, _("%s:%d: error: undefined variable `%s'\n"),
-	   defs_file, linenum, sym->u.sym);
+	   filename, linenum, sym->u.sym);
   exit (1);
 
   /* NOTREACHED */
@@ -561,11 +577,12 @@ lookup_var (env, ns, sym, linenum)
 
 
 static void
-set_var (env, ns, sym, val, linenum)
+set_var (env, ns, sym, val, filename, linenum)
      Environment *env;
      StringHashPtr ns;
      Node *sym;
      Node *val;
+     char *filename;
      unsigned int linenum;
 {
   Node *n;
@@ -589,17 +606,18 @@ set_var (env, ns, sym, val, linenum)
 
   /* Couldn't set value for variable. */
   fprintf (stderr, _("%s:%d: error: couldn't set variable `%s'\n"),
-	   defs_file, linenum, sym->u.sym);
+	   filename, linenum, sym->u.sym);
   exit (1);
   /* NOTREACHED */
 }
 
 
 static Node *
-calculate_binary (l, r, type, linenum)
+calculate_binary (l, r, type, filename, linenum)
      Node *l;
      Node *r;
      ExprType type;
+     char *filename;
      unsigned int linenum;
 {
   Node *n = NULL;
@@ -739,7 +757,7 @@ calculate_binary (l, r, type, linenum)
 	{
 	  fprintf (stderr,
 		   _("%s:%d: error: expression between illegal types\n"),
-		   defs_file, linenum);
+		   filename, linenum);
 	  exit (1);
 	}
       break;
@@ -783,7 +801,8 @@ eval_expr (expr, env)
       break;
 
     case eSYMBOL:
-      n = lookup_var (env, ns_vars, expr->u.node, expr->linenum);
+      n = lookup_var (env, ns_vars, expr->u.node, expr->filename,
+		      expr->linenum);
       node_reference (n);
       return n;
       break;
@@ -887,7 +906,8 @@ eval_expr (expr, env)
       else if (strhash_get (ns_prims, n->u.sym, strlen (n->u.sym),
 			    (void **) &prim))
 	{
-	  n = (*prim) (n->u.sym, expr->u.fcall.args, env, expr->linenum);
+	  n = (*prim) (n->u.sym, expr->u.fcall.args, env, expr->filename,
+		       expr->linenum);
 	  return n;
 	}
       else
@@ -900,7 +920,8 @@ eval_expr (expr, env)
 
     case eASSIGN:
       n = eval_expr (expr->u.assign.expr, env);
-      set_var (env, ns_vars, expr->u.assign.sym, n, expr->linenum);
+      set_var (env, ns_vars, expr->u.assign.sym, n, expr->filename,
+	       expr->linenum);
 
       node_reference (n);
       return n;
@@ -911,24 +932,25 @@ eval_expr (expr, env)
     case eMULASSIGN:
     case eDIVASSIGN:
       n = eval_expr (expr->u.assign.expr, env);
-      n2 = lookup_var (env, ns_vars, expr->u.assign.sym, expr->linenum);
+      n2 = lookup_var (env, ns_vars, expr->u.assign.sym, expr->filename,
+		       expr->linenum);
 
       switch (expr->type)
 	{
 	case eADDASSIGN:
-	  n2 = calculate_binary (n2, n, ePLUS, expr->linenum);
+	  n2 = calculate_binary (n2, n, ePLUS, expr->filename, expr->linenum);
 	  break;
 
 	case eSUBASSIGN:
-	  n2 = calculate_binary (n2, n, eMINUS, expr->linenum);
+	  n2 = calculate_binary (n2, n, eMINUS, expr->filename, expr->linenum);
 	  break;
 
 	case eMULASSIGN:
-	  n2 = calculate_binary (n2, n, eMULT, expr->linenum);
+	  n2 = calculate_binary (n2, n, eMULT, expr->filename, expr->linenum);
 	  break;
 
 	case eDIVASSIGN:
-	  n2 = calculate_binary (n2, n, eDIV, expr->linenum);
+	  n2 = calculate_binary (n2, n, eDIV, expr->filename, expr->linenum);
 	  break;
 
 	default:
@@ -936,7 +958,8 @@ eval_expr (expr, env)
 	  abort ();
 	  break;
 	}
-      set_var (env, ns_vars, expr->u.assign.sym, n2, expr->linenum);
+      set_var (env, ns_vars, expr->u.assign.sym, n2, expr->filename,
+	       expr->linenum);
 
       node_free (n);
       node_reference (n2);
@@ -948,13 +971,14 @@ eval_expr (expr, env)
       sn.type = nINTEGER;
       sn.u.integer = 1;
 
-      n2 = lookup_var (env, ns_vars, expr->u.node, expr->linenum);
+      n2 = lookup_var (env, ns_vars, expr->u.node, expr->filename,
+		       expr->linenum);
       node_reference (n2);
 
       n = calculate_binary (n2, &sn,
 			    expr->type == ePOSTFIXADD ? ePLUS : eMINUS,
-			    expr->linenum);
-      set_var (env, ns_vars, expr->u.node, n, expr->linenum);
+			    expr->filename, expr->linenum);
+      set_var (env, ns_vars, expr->u.node, n, expr->filename, expr->linenum);
 
       return n2;
       break;
@@ -964,11 +988,12 @@ eval_expr (expr, env)
       sn.type = nINTEGER;
       sn.u.integer = 1;
 
-      n = lookup_var (env, ns_vars, expr->u.node, expr->linenum);
+      n = lookup_var (env, ns_vars, expr->u.node, expr->filename,
+		      expr->linenum);
       n = calculate_binary (n, &sn,
 			    expr->type == ePREFIXADD ? ePLUS : eMINUS,
-			    expr->linenum);
-      set_var (env, ns_vars, expr->u.node, n, expr->linenum);
+			    expr->filename, expr->linenum);
+      set_var (env, ns_vars, expr->u.node, n, expr->filename, expr->linenum);
 
       node_reference (n);
       return n;
@@ -980,7 +1005,7 @@ eval_expr (expr, env)
 	{
 	  fprintf (stderr,
 		   _("%s:%d: error: illegal lvalue for assignment\n"),
-		   defs_file, expr->linenum);
+		   expr->filename, expr->linenum);
 	  exit (1);
 	}
       n2 = eval_expr (expr->u.arrayassign.expr2, env);
@@ -988,13 +1013,13 @@ eval_expr (expr, env)
 	{
 	  fprintf (stderr,
 		   _("%s:%d: error: array reference index is not integer\n"),
-		   defs_file, expr->linenum);
+		   expr->filename, expr->linenum);
 	  exit (1);
 	}
       if (n2->u.integer < 0)
 	{
 	  fprintf (stderr, _("%s:%d: error: negative array reference index\n"),
-		   defs_file, expr->linenum);
+		   expr->filename, expr->linenum);
 	  exit (1);
 	}
 
@@ -1045,7 +1070,7 @@ eval_expr (expr, env)
 	    {
 	      fprintf (stderr,
 		       _("%s:%d: error: illegal rvalue for string assignment\n"),
-		       defs_file, expr->linenum);
+		       expr->filename, expr->linenum);
 	      exit (1);
 	    }
 
@@ -1064,7 +1089,7 @@ eval_expr (expr, env)
 	{
 	  fprintf (stderr,
 		   _("%s:%d: error: illegal type for array reference\n"),
-		   defs_file, expr->linenum);
+		   expr->filename, expr->linenum);
 	  exit (1);
 	}
       n2 = eval_expr (expr->u.arrayref.expr2, env);
@@ -1072,7 +1097,7 @@ eval_expr (expr, env)
 	{
 	  fprintf (stderr,
 		   _("%s:%d: error: array reference index is not integer\n"),
-		   defs_file, expr->linenum);
+		   expr->filename, expr->linenum);
 	  exit (1);
 	}
       if (n2->u.integer < 0
@@ -1081,7 +1106,7 @@ eval_expr (expr, env)
 	{
 	  fprintf (stderr,
 		   _("%s:%d: error: array reference index out of rance\n"),
-		   defs_file, expr->linenum);
+		   expr->filename, expr->linenum);
 	  exit (1);
 	}
 
@@ -1146,7 +1171,7 @@ eval_expr (expr, env)
       l = eval_expr (expr->u.op.left, env);
       r = eval_expr (expr->u.op.right, env);
 
-      n = calculate_binary (l, r, expr->type, expr->linenum);
+      n = calculate_binary (l, r, expr->type, expr->filename, expr->linenum);
 
       node_free (l);
       node_free (r);
@@ -1177,7 +1202,8 @@ eval_statement (stmt, env, return_seen)
       break;
 
     case sDEFSUB:
-      define_sub (stmt->u.defsub.name, stmt->u.defsub.closure, stmt->linenum);
+      define_sub (stmt->u.defsub.name, stmt->u.defsub.closure,
+		  stmt->filename, stmt->linenum);
       break;
 
     case sBLOCK:
@@ -1290,3 +1316,111 @@ eval_statement_list (lst, env, return_seen)
 
   return n;
 }
+
+
+void
+load_states_file (name)
+     char *name;
+{
+  Node *n;
+  int return_seen = 0;
+
+  yyin_name = xstrdup (name);
+  linenum = 1;
+
+  yyin = fopen (yyin_name, "r");
+  if (yyin == NULL)
+    {
+      fprintf (stderr, _("%s: couldn't open definition file `%s': %s\n"),
+	       program, yyin_name, strerror (errno));
+      exit (1);
+    }
+
+
+  yyparse ();
+  fclose (yyin);
+
+  /* Evaluate all top-level statements. */
+  n = eval_statement_list (global_stmts, NULL, &return_seen);
+  node_free (n);
+
+  /* Reset the global statements to an empty list. */
+  global_stmts = list ();
+}
+
+
+int
+autoload_file (name)
+     char *name;
+{
+  char *start;
+  unsigned int len;
+  char *cp;
+  char *buf = NULL;
+  unsigned int buflen = 1024;
+  unsigned int name_len;
+  struct stat stat_st;
+  int result = 0;
+
+  name_len = strlen (name);
+  buf = xmalloc (buflen);
+
+  for (start = path; start; start = cp)
+    {
+      cp = strchr (start, PATH_SEPARATOR);
+      if (cp)
+	{
+	  len = cp - start;
+	  cp++;
+	}
+      else
+	len = strlen (start);
+
+      if (len + 1 + name_len + 3 + 1 >= buflen)
+	{
+	  buflen = len + 1 + name_len + 3 + 1 + 1024;
+	  buf = xrealloc (buf, buflen);
+	}
+      sprintf (buf, "%.*s/%s.st", len, start, name);
+
+      if (stat (buf, &stat_st) == 0)
+	{
+	  if (verbose)
+	    fprintf (stderr,
+		     _("%s: autoloading `%s' from `%s'\n"),
+		     program, name, buf);
+	  load_states_file (buf);
+	  result = 1;
+	  break;
+	}
+    }
+
+  xfree (buf);
+
+  return result;
+}
+
+
+State *
+lookup_state (name)
+     char *name;
+{
+  State *state;
+  int retry_count = 0;
+
+  while (1)
+    {
+      if (strhash_get (ns_states, name, strlen (name), (void **) &state))
+	return state;
+
+      if (retry_count > 0)
+	break;
+
+      /* Try to autoload the state. */
+      autoload_file (name);
+      retry_count++;
+    }
+
+  /* No luck. */
+  return NULL;
+}
